@@ -38,7 +38,7 @@ from requests.adapters import HTTPAdapter
 from requests.packages.urllib3.util.retry import Retry
 
 
-VERSION = '2.0.1'
+VERSION = '2.0.2'
 MANIFEST_LOCKH = None
 REPO_LOCKH = dict()
 GITBIN = '/usr/bin/git'
@@ -104,19 +104,20 @@ def git_newer_than(minver):
     return version.parse(ver) >= version.parse(minver)
 
 
-def run_shell_command(cmdargs, stdin=None):
+def run_shell_command(cmdargs, stdin=None, decode=True):
     logger.debug('Running: %s', ' '.join(cmdargs))
 
     child = subprocess.Popen(cmdargs, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
     output, error = child.communicate(input=stdin)
 
-    output = output.decode().strip()
-    error = error.decode().strip()
+    if decode:
+        output = output.decode().strip()
+        error = error.decode().strip()
 
     return child.returncode, output, error
 
 
-def run_git_command(fullpath, args, stdin=None):
+def run_git_command(fullpath, args, stdin=None, decode=True):
     if 'GITBIN' in os.environ:
         _git = os.environ['GITBIN']
     else:
@@ -131,7 +132,7 @@ def run_git_command(fullpath, args, stdin=None):
     else:
         cmdargs = [_git, '--no-pager'] + args
 
-    return run_shell_command(cmdargs, stdin)
+    return run_shell_command(cmdargs, stdin, decode=decode)
 
 
 def _lockname(fullpath):
@@ -226,7 +227,7 @@ def get_repo_obj_info(fullpath):
     return obj_info
 
 
-def get_repo_defs(toplevel, gitdir, usenow=False):
+def get_repo_defs(toplevel, gitdir, usenow=False, ignorerefs=None):
     fullpath = os.path.join(toplevel, gitdir.lstrip('/'))
     description = None
     try:
@@ -276,7 +277,7 @@ def get_repo_defs(toplevel, gitdir, usenow=False):
     # "state fingerprint" -- basically the output of "git show-ref | sha1sum".
     # git show-ref output is deterministic and should accurately list all refs
     # and their relation to heads/tags/etc.
-    fingerprint = get_repo_fingerprint(toplevel, gitdir, force=True)
+    fingerprint = get_repo_fingerprint(toplevel, gitdir, force=True, ignorerefs=ignorerefs)
     # Record it in the repo for other use
     set_repo_fingerprint(toplevel, gitdir, fingerprint)
     repoinfo = {
@@ -661,7 +662,7 @@ def get_forkgroups(obstdir, toplevel):
     return forkgroups
 
 
-def get_repo_fingerprint(toplevel, gitdir, force=False):
+def get_repo_fingerprint(toplevel, gitdir, force=False, ignorerefs=None):
     fullpath = os.path.join(toplevel, gitdir.lstrip('/'))
     if not os.path.exists(fullpath):
         logger.debug('Cannot fingerprint %s, as it does not exist', fullpath)
@@ -679,9 +680,24 @@ def get_repo_fingerprint(toplevel, gitdir, force=False):
             logger.debug('No heads in %s, nothing to fingerprint.', fullpath)
             return None
 
-        # We add the final "\n" to be compatible with cmdline output
-        # of git-show-ref
-        fingerprint = hashlib.sha1(out.encode() + b"\n").hexdigest()
+        if ignorerefs:
+            hasher = hashlib.sha1()
+            for line in out.split('\n'):
+                rhash, rname = line.split(maxsplit=1)
+                ignored = False
+                for ignoreref in ignorerefs:
+                    if fnmatch.fnmatch(rname, ignoreref):
+                        ignored = True
+                        break
+                if ignored:
+                    continue
+                hasher.update(line.encode() + b'\n')
+
+            fingerprint = hasher.hexdigest()
+        else:
+            # We add the final "\n" to be compatible with cmdline output
+            # of git-show-ref
+            fingerprint = hashlib.sha1(out.encode() + b'\n').hexdigest()
 
         # Save it for future use
         if not force:
@@ -911,16 +927,23 @@ def load_config_file(cfgfile):
         sys.stderr.write('       Perhaps this is a grokmirror-1.x config file?\n')
         sys.exit(1)
 
-    toplevel = os.path.realpath(config['core'].get('toplevel'))
+    toplevel = os.path.realpath(os.path.expanduser(config['core'].get('toplevel')))
     if not os.access(toplevel, os.W_OK):
         logger.critical('Toplevel %s does not exist or is not writable', toplevel)
         sys.exit(1)
+    # Just in case we did expanduser
+    config['core']['toplevel'] = toplevel
 
     obstdir = config['core'].get('objstore', None)
     if obstdir is None:
         obstdir = os.path.join(toplevel, 'objstore')
         config['core']['objstore'] = obstdir
 
+    # Handle some other defaults
+    manifile = config['core'].get('manifest')
+    if not manifile:
+        config['core']['manifest'] = os.path.join(toplevel, 'manifest.js.gz')
+
     fstat = os.stat(cfgfile)
     # stick last config file modification date into the config object,
     # so we can catch config file updates
@@ -988,7 +1011,7 @@ def init_logger(subcommand, logfile, loglevel, verbose):
     logger.setLevel(logging.DEBUG)
 
     if logfile:
-        ch = logging.handlers.WatchedFileHandler(logfile)
+        ch = logging.handlers.WatchedFileHandler(os.path.expanduser(logfile))
         formatter = logging.Formatter(subcommand + '[%(process)d] %(asctime)s - %(levelname)s - %(message)s')
         ch.setFormatter(formatter)
         ch.setLevel(loglevel)
